#include <avr/io.h>

// Librairie AtMega pour PWM 

#include "..\ladder.h"
#include "PwmLib.h"


// compute prediv coefficient
int PWM_Prediv(long fmcu, long fpwm, int csmax)
	{
    long p = (fmcu /256) / fpwm;		// predivision theorique

    if (csmax == 0) return (0);    	// desactivera la pwm

    if (csmax == 5)			// pwm a 5 possibilites de predivision
    	{
	    if (p >= 1024)		return(5);		// cs (tres) approximatif
    	if (p >= 256)		return(4);
		if (p >= 64)		return(3);
		if (p >= 8)			return(2);
		if (p >= 1)			return(1);
		return(0);
        }
    else          			// csmax = 7
    	{
    	if (p >= 1024)		return(7);		// cs (tres) approximatif
    	if (p >= 256)		return(6);
		if (p >= 128)		return(5);
		if (p >= 64)		return(4);
		if (p >= 32)		return(3);
		if (p >= 8)			return(2);
		if (p >= 1)			return(1);
		return(0);
        }
    }


#ifdef LDTARGET_atmega16
// set fast PWM mode with non-inverted output
void PWM_Init(int canal, long fmcu, long fpwm, int resol, int csmax)
	{
    int cs= PWM_Prediv(fmcu, fpwm, csmax);

    if (canal == 0x00)
		{
		TCCR0 = (1 << WGM00) | (1 << WGM01) | (1 << COM01) | cs;	// 8 bits
		// set OC0 pin as output
		DDRB |= (1 << 3);

        OCR0 = 0;
		}
	if (canal == 0x1A)
		{
		TCCR1A |= (1 << WGM10) | (1 << WGM11) | (1 << COM1A1);
		TCCR1B = (1 << WGM12)  | cs;								// 10 bits
		// set OC0 pin as output
		DDRD |= (1 << 5);

        OCR1A = 0;
		}
	if (canal == 0x1B)
		{
		TCCR1A |= (1 << WGM10) | (1 << WGM11) | (1 << COM1B1);
		TCCR1B = (1 << WGM12)  | cs;								// 10 bits
		// set OC0 pin as output
		DDRD |= (1 << 4);

        OCR1B = 0;
		}
    if (canal == 0x02)
    	{
		TCCR2 = (1 << WGM20) | (1 << WGM21) | (1 << COM21) | cs;	// 8 bits
		// set OC0 pin as output
		DDRD |= (1 << 7);

		OCR2= 0;
        }
	}

// modify duty
void PWM_Set(int canal, unsigned percent, int resol)
	{
    unsigned value= 0;
    if (resol == 8)
        value= (255 * percent) / 100;		// PWM 8 bit
    else
        value= (1023 * percent) / 100;		// PWM 10 bit

    if (canal == 0x00) OCR0= value;
	if (canal == 0x1A) OCR1A= value;
	if (canal == 0x1B) OCR1B= value;
    if (canal == 0x02) OCR2= value;
	}
#endif

#ifdef LDTARGET_atmega328
// set fast PWM mode with non-inverted output
void PWM_Init(int canal, long fmcu, long fpwm, int resol, int csmax)
	{
    int cs= PWM_Prediv(fmcu, fpwm, csmax);

	if (canal == 0x0A)
		{
		// set fast PWM mode with non-inverted output
		TCCR0A |= (1 << WGM00) | (1 << WGM01) | (1 << COM0A1);
		TCCR0B = cs;
		// set OC pin as output
		DDRD |= (1 << 6);

		OCR0A = 0;
		}
	if (canal == 0x0B)
		{
		// set fast PWM mode with non-inverted output
		TCCR0A |= (1 << WGM00) | (1 << WGM01) | (1 << COM0B1);
		TCCR0B = cs;
		// set OC pin as output
		DDRD |= (1 << 5);

		OCR0B = 0;
		}
	if (canal == 0x2A)
		{
		// set fast PWM mode with non-inverted output
		TCCR2A |= (1 << WGM20) | (1 << WGM21) | (1 << COM2A1);
		TCCR2B = cs;
		// set OC pin as output
		DDRB |= (1 << 3);

		OCR2A= 0;
		}
	if (canal == 0x2B)
		{
		// set fast PWM mode with non-inverted output
		TCCR2A |= (1 << WGM20) | (1 << WGM21) | (1 << COM2B1);
		TCCR2B = cs;
		// set OC pin as output
		DDRD |= (1 << 3);

		OCR2A= 0;
		}
	}

// modify duty
void PWM_Set(int canal, unsigned percent, int resol)
	{
    unsigned value= 0;
    if (resol == 8)
        value= (255 * percent) / 100;	  	// PWM 8 bit
    else
        value= (1023 * percent) / 100;		// PWM 10 bit

	if (canal == 0x0A)	OCR0A= value;
	if (canal == 0x0B)	OCR0B= value;
	if (canal == 0x2A)	OCR2A= value;
    if (canal == 0x2B)	OCR2B= value;
	}
#endif


